@@ -3,6 +3,7 @@
 
 #include <limits>
 #include <cmath>
+#include <vector>
 
 using namespace std;
 
@@ -13,7 +14,8 @@ class errmonitreal {  // a real value of type T, with a first order estimate of
 	public:
 		errmonitreal(const T v,const T e,const bool dpec) : val(v) , er(e) {  }	// Constructor	
 		errmonitreal(const T v,const T e) : val(v) , er(e) {  }	// Constructor	
-		errmonitreal(const T v) : val(v), er(0.)	{  }	// Constructor	
+//		errmonitreal(const T v)	: val(v), er(0.)	{  }	// Constructor
+		errmonitreal(const T v)	: val(v), er(RNDERR)	{  }	// Constructor
 		errmonitreal(void)	:	val(0.), er(0.)	{  }	// Constructor
 		errmonitreal(const errmonitreal<T>& org)
 		:  val(org.value()) , er(org.err())  {   }	// Copy Constructor	
@@ -27,6 +29,7 @@ class errmonitreal {  // a real value of type T, with a first order estimate of
 		const	T	value(void)	const	{ return val; }	// Return value
 		const	T	err(void)	const	{ return er; }	// Return estimate of roundig error		
 		const static T RNDERR;  // Machine precision for basic type
+		const static T MINRLPOSNB;  // Minimum reliably positive number for basic type
 		static bool dropec;  // Drop error checking when set to true
 	private:
 		T  val;	// computed value
@@ -37,6 +40,7 @@ template <class T>
 bool errmonitreal<T>::dropec = false; // Error checking active by default 
 
 template<class T> const T errmonitreal<T>::RNDERR = std::numeric_limits<T>::epsilon(); 
+template<class T> const T errmonitreal<T>::MINRLPOSNB = 2.5*RNDERR; 
 
 template<class T> const errmonitreal<T> operator+(const errmonitreal<T>& lhs,const errmonitreal<T>& rhs);
 template<class T> const errmonitreal<T> operator-(const errmonitreal<T>& lhs,const errmonitreal<T>& rhs);
@@ -45,6 +49,7 @@ template<class T> inline const errmonitreal<T> operator/(const errmonitreal<T>&
 
 template<class T> bool errcheck(const errmonitreal<T> *realp,const double tol);
 template<class T> bool errcheck(errmonitreal<T> **realpl,const double tol,const int nreals);
+template<class T> bool errcheck(vector<errmonitreal<T> *>& realpl,const double tol,const int nreals);
 
 
 template<class T> inline bool errcheck(const errmonitreal<T> *realp,const double tol)
@@ -59,6 +64,11 @@ template<class T> inline bool errcheck(errmonitreal<T> **realpl,const double tol
 	return true;
 };
 
+template<class T> inline bool errcheck(vector<errmonitreal<T> *>& realpl,const double tol,const int nreals)
+{ 
+	errcheck(&realpl[0],tol,nreals);
+};
+
 inline bool errcheck(double *realp,const double tol)
 { 
 	return true;
@@ -85,8 +95,11 @@ errmonitreal<T>& errmonitreal<T>::operator+=(const errmonitreal<T>& rhs)
 	T res = val + rhs.value();
 	if (!dropec) {
 		T absres = std::fabs(res);
-		if (absres < std::numeric_limits<T>::min()) er = std::numeric_limits<T>::max();
-		else er = (fabs(val)*er+fabs(rhs.value())*rhs.err())/absres+RNDERR;
+		if (er > 0. || rhs.err() > 0.)  {
+			if (absres < MINRLPOSNB) er = std::numeric_limits<T>::max();
+			else er = (fabs(val)*er+fabs(rhs.value())*rhs.err())/absres+RNDERR;
+		}
+		else er = RNDERR;
 	}
 	val = res;
 	return *this;
@@ -98,8 +111,11 @@ errmonitreal<T>& errmonitreal<T>::operator-=(const errmonitreal<T>& rhs)
 	T res = val - rhs.value();;
 	if (!dropec) {
 		T absres = std::fabs(res);
-		if (absres < std::numeric_limits<T>::min()) er = std::numeric_limits<T>::max();
-		else er = (fabs(val)*er+fabs(rhs.value())*rhs.err())/absres+RNDERR;
+		if (er > 0. || rhs.err() > 0.)  {
+			if (absres < MINRLPOSNB) er = std::numeric_limits<T>::max();
+			else er = (fabs(val)*er+fabs(rhs.value())*rhs.err())/absres+RNDERR;
+		}
+		else er = RNDERR;
 	}
 	val = res;
 	return *this;
@@ -109,7 +125,13 @@ template<class T>
 errmonitreal<T>& errmonitreal<T>::operator*=(const errmonitreal<T>& rhs)
 {	
 	val *= rhs.value();
-	if (!dropec) er += rhs.err() + RNDERR;
+	if (!dropec) {
+		T absres = std::fabs(val);
+		if (absres > 0. && absres < MINRLPOSNB) er = std::numeric_limits<T>::max();
+		else er += rhs.err() + RNDERR;
+//		er += rhs.err() + RNDERR;
+
+	}
 	return *this;
 }
 
@@ -117,7 +139,12 @@ template<class T>
 errmonitreal<T>& errmonitreal<T>::operator/=(const errmonitreal<T>& rhs)
 {	
 	val /= rhs.value();
-	if (!dropec) er += rhs.err() + RNDERR;
+	if (!dropec) {
+		T absres = std::fabs(val);
+		if (absres > 0. && absres < MINRLPOSNB) er = std::numeric_limits<T>::max();
+		else er += rhs.err() + RNDERR;
+//		er += rhs.err() + RNDERR;
+	}
 	return *this;
 }
 
@@ -127,7 +154,10 @@ const errmonitreal<T> operator+(const errmonitreal<T>& lhs,const errmonitreal<T>
 	T res = lhs.value() + rhs.value();
 	if (!errmonitreal<T>::dropec) {
 		T er,absres=std::fabs(res);
-		if (absres < std::numeric_limits<T>::min()) er = std::numeric_limits<T>::max();
+		if (lhs.err() > 0. || rhs.err() > 0.)  {
+			if (absres < errmonitreal<T>::MINRLPOSNB) er = std::numeric_limits<T>::max();
+			else er = (std::fabs(lhs.value())*lhs.err()+std::fabs(rhs.value())*rhs.err())/absres + errmonitreal<T>::RNDERR;
+		}
 		else er = (std::fabs(lhs.value())*lhs.err()+std::fabs(rhs.value())*rhs.err())/absres + errmonitreal<T>::RNDERR;
 		return errmonitreal<T>(res,er);
 	}
@@ -140,7 +170,10 @@ const errmonitreal<T> operator-(const errmonitreal<T>& lhs,const errmonitreal<T>
 	T res = lhs.value()-rhs.value();
 	if (!errmonitreal<T>::dropec) {
 		T er,absres=std::fabs(res);
-		if (absres < std::numeric_limits<T>::min()) er = std::numeric_limits<T>::max();
+		if (lhs.err() > 0. || rhs.err() > 0.)  {
+			if (absres < errmonitreal<T>::MINRLPOSNB) er = std::numeric_limits<T>::max();
+			else er = (std::fabs(lhs.value())*lhs.err()+std::fabs(rhs.value())*rhs.err())/absres + errmonitreal<T>::RNDERR;
+		}
 		else er = (std::fabs(lhs.value())*lhs.err()+std::fabs(rhs.value())*rhs.err())/absres + errmonitreal<T>::RNDERR;
 		return errmonitreal<T>(res,er);
 	}
@@ -150,15 +183,29 @@ const errmonitreal<T> operator-(const errmonitreal<T>& lhs,const errmonitreal<T>
 template<class T>
 const errmonitreal<T> operator*(const errmonitreal<T>& lhs,const errmonitreal<T>& rhs)
 {	
-	if (!errmonitreal<T>::dropec) return errmonitreal<T>(lhs.value()*rhs.value(),lhs.err()+rhs.err()+errmonitreal<T>::RNDERR);
-	else return errmonitreal<T>(lhs.value()*rhs.value());
+	T res = lhs.value()*rhs.value();
+	if (!errmonitreal<T>::dropec) {
+		T er,absres=std::fabs(res);
+		if (absres > 0. && absres < errmonitreal<T>::MINRLPOSNB) er = std::numeric_limits<T>::max();
+		else er = lhs.err() + rhs.err() + errmonitreal<T>::RNDERR;
+//		er = lhs.err() + rhs.err() + errmonitreal<T>::RNDERR;
+		return errmonitreal<T>(res,er);
+	}
+	else return errmonitreal<T>(res);
 }
 
 template<class T>
 const errmonitreal<T> operator/(const errmonitreal<T>& lhs,const errmonitreal<T>& rhs)
 {	
-	if (!errmonitreal<T>::dropec) return errmonitreal<T>(lhs.value()/rhs.value(),lhs.err()+rhs.err()+errmonitreal<T>::RNDERR);
-	else return errmonitreal<T>(lhs.value()/rhs.value());
+	T res = lhs.value()/rhs.value();
+	if (!errmonitreal<T>::dropec) {
+		T er,absres=std::fabs(res);
+		if (absres > 0. && absres < errmonitreal<T>::MINRLPOSNB)  er = std::numeric_limits<T>::max();
+		else er = lhs.err() + rhs.err() + errmonitreal<T>::RNDERR;
+//		er = lhs.err() + rhs.err() + errmonitreal<T>::RNDERR;
+		return errmonitreal<T>(res,er);
+	}
+	else return errmonitreal<T>(res);
 }
 
 }
